# $Id$

package XML::Atom::Server::Lite::Feed;
use strict;
use XML::Atom::Server::Lite qw/qquote/;

sub new {
    my $class = shift;
    my $self = bless {}, $class;
    $self->{title} = '';
    $self->{link} = [];
    $self->{entries} = [];
    $self->{author} = [];
    $self->{updated} = '';
    $self;
}

sub title {
    my $self = shift;
    if (@_) { $self->{title} = $_[0] } else { $self->{title}||'' };
}

sub author {
    my $self = shift;
    if (@_) { $self->{author} = \@_ } else { $self->{author}||() };
}

sub link {
    my $self = shift;
    if (@_) {
        if (ref($_[0]) eq 'ARRAY') {
            $self->{link} = $_[0];
        } else {
            push @{$self->{link}}, $_[0];
        }
    } else { $self->{link}||(); };
}

sub updated {
    my $self = shift;
    if (@_) { $self->{updated} = \@_ } else { $self->{updated}||() };
}

sub entries {
    my $self = shift;
    if (@_) {
        if (ref($_[0]) eq 'ARRAY') {
            $self->{entries} = $_[0];
        } else {
            push @{$self->{entries}}, $_[0];
        }
    } else { $self->{entries}||(); };
}

sub add_entry {
    my $self = shift;
    my($entry, $opt) = @_;
    push @{$self->{entries}}, $entry;
}

sub to_xml {
    my $tree = shift;
    my $xml;
    if ($tree->{name}) {
        $xml .= "<".$tree->{name};
        for my $attr (keys %{$tree->{attributes}}) {
            $xml .= " $attr=".qquote($tree->{attributes}->{$attr});
        }
    } elsif ($tree->{content}) {
        $xml .= $tree->{content};
    }
    if ($tree->{children}) {
        $xml .= ">" if $tree->{name};
        foreach my $item (@{$tree->{children}}) {
            $xml .= to_xml($item);
        }
        $xml .= "</".$tree->{name}.">" if $tree->{name};
    } elsif ($tree->{name}) {
        $xml .= "/>";
    }
    $xml;
}

sub as_xml {
    my $self = shift;
    my $xml = "";
    $xml .= "<feed xmlns=\"http://purl.org/atom/ns#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\">\n";
    $xml .= " <title>"
           . XML::Atom::Server::Lite::encode_xml($self->{title})
           . "</title>\n" if $self->{title};
    for my $link (@{$self->{link}}) {
        $xml .= " <link"
             .  " rel=".XML::Atom::Server::Lite::qquote($link->{rel})
             .  " title=".XML::Atom::Server::Lite::qquote($link->{title})
             .  " type=".XML::Atom::Server::Lite::qquote($link->{type})
             .  " href=".XML::Atom::Server::Lite::qquote($link->{href})."/>\n";
    }
    $xml .= " <updated>"
           . XML::Atom::Server::Lite::encode_xml($self->{updated})
           . "</updated>\n" if $self->{updated};
    for my $entry (@{$self->{entries}}) {
        $xml .= $entry->to_xml;
    }
    $xml .= "</feed>";
}

1;
