# sb::Admin::CommentOption
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::CommentOption;
use strict;
use sb::Utils ();
use sb::Admin::List ();
use base qw( sb::Admin::List );
use vars qw( $VERSION );
$VERSION = '0.06';
# 0.06 [2007/12/03] modified to segment the setting into comments and trackbacks
# 0.05 [2007/11/28] modified to use sb::Utils->remove_duplicates
# 0.04 [2007/09/26] enabled $self->conf->save, fixed a simple syntax bug in _update
# 0.03 [2007/09/17] ported from Serene Bach 2.* (Refusal)
# 0.02 [2007/02/16] changed _open_refusal_setting to show conf_spamstat always
# 0.01 [2006/09/10] changed _save_refusal_setting and _open_refusal_setting to handle conf_spamstat
# 0.00 [2005/04/09] generate
# --------------------------------------
# // defines
sub TEMPLATE     (){ 'refusal.html' };
sub CONFIG_LEVEL (){ 1 }
# --------------------------------------
# // public functions - for sub class
sub check_configurability
{
	my $self = shift;
	return $self->has_permission('level'=>CONFIG_LEVEL);
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	return ($self->regi) ? _update($self,@_) : _display($self,@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my $msg = undef;
	my @list = split("\n",$self->conf->value('conf_ip_ban'));
	SWITCH_MODE: {
		$_ = $self->regi();
		/^spam$/ && do
		{ # change settings against spams
			if ($self->mode eq 'comment')
			{ # for messages
				$self->conf->value('conf_spamlevel'=>int($self->cgi->value('refuse_level')));
				if ($self->cgi->value('refuse_cookie') ne '')
				{
					$self->conf->value('conf_spamid'=>$self->cgi->value('refuse_cookie'));
				}
				if ($self->cgi->value('refuse_word') ne '')
				{
					$self->conf->value('conf_spamword'=>sb::Text->new($self->cgi->value('refuse_word'))->entitized());
				}
				$self->conf->value('conf_spamstat' => ($self->cgi->value('refuse_status') eq '') ? 0 : 1 );
			}
			else
			{ # for trackbacks
				$self->conf->value('conf_spamtblevel'=>int($self->cgi->value('refuse_level')));
				if ($self->cgi->value('refuse_word') ne '')
				{
					$self->conf->value('conf_spamtbword'=>sb::Text->new($self->cgi->value('refuse_word'))->entitized());
				}
				$self->conf->value('conf_spamtbstat' => ($self->cgi->value('refuse_status') eq '') ? 0 : 1 );
			}
			last SWITCH_MODE;
		};
		/^addip$/ && do
		{ # add new ip
			if ($self->cgi->value('refuse_ip') ne '')
			{
				unshift(@list,$self->cgi->value('refuse_ip'));
				@list = sb::Utils->remove_duplicates(@list);
				$self->conf->value('conf_ip_ban' => join("\n",@list));
			}
			last SWITCH_MODE;
		};
		/^delip$/ && do
		{ # delete selected ips
			my @new_list = ();
			foreach my $ip (@list)
			{
				my $flag = undef;
				foreach my $del ( split("\0",$self->cgi->value('sel')) )
				{
					$flag = 1 if ($ip eq $del);
					last if ($flag);
				}
				push(@new_list,$ip) if (!$flag);
			}
			$self->conf->value('conf_ip_ban' => join("\n",@new_list));
			last SWITCH_MODE;
		};
	} # end of SWITCH_MODE
	$self->conf->save($self->conf->config_file());
	# we call _display directly here, because $self is an instance of 
	# sb::Admin::Message or Trackback.  
	return _display($self,'message'=>$msg);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		@_
	);
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	my $path = $self->script_path . '?__mode=' . $self->mode . '&amp;disptype=host&amp;dispword=';
	my $level = ($self->mode eq 'comment')
		? $self->conf->value('conf_spamlevel')
		: $self->conf->value('conf_spamtblevel');
	my $count = ($self->mode eq 'comment')
		? $self->_count_host('Message')
		: $self->_count_host('Trackback');
	my @list = split("\n",$self->conf->value('conf_ip_ban'));
	for my $i (0 .. $#list)
	{
		my $check = $list[$i];
		my $cnt = 0;
		map { (/^$check/) ? $cnt += $count->{$_} : 1 } keys(%{$count});
		$temp->num($i);
		$temp->tag('sb_refuse_ip'  => $check);
		$temp->tag('sb_refuse_obj' => ($cnt > 0) ? '<a href="' . $path . $check . '">' . $cnt . '</a>' : $cnt);
		$temp->tag('sb_list_class' => ($i % 2) ? 'odd' : 'even');
	}
	$temp->block('sb_refuse_list' => $#list + 1);
	$temp->num(0);
	$temp->tag('sb_refuse_level_' . $level => 'selected="selected"');
	if ($level > -1)
	{
		my $check = ($self->mode eq 'comment')
			? $self->conf->value('conf_spamstat')
			: $self->conf->value('conf_spamtbstat');
		$temp->tag('sb_refuse_status'=>'checked="checked"') if ($check);
		$temp->block('sb_refuse_status'=>1);
	}
	if ($level == 2 and $self->mode eq 'comment')
	{
		$temp->tag('sb_refuse_cookie'=>sb::Text->new($self->conf->value('conf_spamid'))->entitized());
		$temp->block('sb_refuse_cookie'=>1);
	}
	if ($level >= 4)
	{
		my $word = ($self->mode eq 'comment')
			? $self->conf->value('conf_spamword')
			: $self->conf->value('conf_spamtbword');
		$temp->tag('sb_refuse_word'=>$word);
		$temp->block('sb_refuse_word'=>1);
	}
	$temp->num(0);
	$temp->tag('sb_selected_modename'=>sb->string('mode_' . $self->mode));
	$temp->tag('sb_selected_mode'=>$self->mode);
	if ($self->mode eq 'comment')
	{
		$temp->block('sb_message_typelist'=>1);
		$temp->block('sb_message_description'=>1);
	}
	else
	{
		$temp->block('sb_trackback_typelist'=>1);
		$temp->block('sb_trackback_description'=>1);
	}
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - display
sub _count_host
{
	my $self = shift;
	my $target = shift;
	my %count = ();
	if ($target)
	{
		my @objs = sb::Data->load($target, 'limit'=>'host');
		map { ($_->host) ? $count{$_->host}++ : 1 } @objs;
	}
	return \%count;
}
1;
__END__
